@@ -15,10 +15,7 @@ from sphinx.util import logging
 import nbconvert
 from nbformat import NotebookNode
 
-from jupyter_sphinx.ast import (
-    JupyterWidgetViewNode,
-    strip_latex_delimiters,
-)
+from jupyter_sphinx.ast import strip_latex_delimiters, JupyterWidgetViewNode
 from jupyter_sphinx.utils import sphinx_abs_dir
 
 from myst_parser.main import default_parser, MdParserConfig
@@ -28,7 +25,6 @@ from .nodes import CellOutputBundleNode
 LOGGER = logging.getLogger(__name__)
 
 WIDGET_VIEW_MIMETYPE = "application/vnd.jupyter.widget-view+json"
-MYST_META_KEY = "myst"
 
 
 def get_default_render_priority(builder: str) -> Optional[List[str]]:
@@ -161,7 +157,6 @@ class CellOutputRendererBase(ABC):
                 else:
                     output_nodes.extend(self.render("stdout", output, idx))
             elif output_type == "error":
-                print("traceback", idx)
                 output_nodes.extend(self.render("traceback", output, idx))
 
             elif output_type in ("display_data", "execute_result"):
@@ -191,11 +186,28 @@ class CellOutputRendererBase(ABC):
             for child in node.traverse():
                 child.source, child.line = location
 
+    def make_warning(self, error_msg: str) -> nodes.system_message:
+        """Raise an exception or generate a warning if appropriate,
+        and return a system_message node"""
+        return self.document.reporter.warning(
+            "output render: {}".format(error_msg),
+            line=self.node.line,
+        )
+
     def make_error(self, error_msg: str) -> nodes.system_message:
         """Raise an exception or generate a warning if appropriate,
         and return a system_message node"""
         return self.document.reporter.error(
-            "output render: {}".format(error_msg), line=self.node.line,
+            "output render: {}".format(error_msg),
+            line=self.node.line,
+        )
+
+    def make_severe(self, error_msg: str) -> nodes.system_message:
+        """Raise an exception or generate a warning if appropriate,
+        and return a system_message node"""
+        return self.document.reporter.severe(
+            "output render: {}".format(error_msg),
+            line=self.node.line,
         )
 
     def add_name(self, node: nodes.Node, name: str):
@@ -272,20 +284,33 @@ class CellOutputRenderer(CellOutputRendererBase):
 
     def render_stderr(self, output: NotebookNode, index: int):
         """Output a container with an unhighlighted literal block."""
+        text = output["text"]
 
-        if "remove-stderr" in self.node.metadata.get("tags", []):
+        if self.env.config.nb_output_stderr == "show":
+            pass
+        elif self.env.config.nb_output_stderr == "remove-warn":
+            self.make_warning(f"stderr was found in the cell outputs: {text}")
+            return []
+        elif self.env.config.nb_output_stderr == "warn":
+            self.make_warning(f"stderr was found in the cell outputs: {text}")
+        elif self.env.config.nb_output_stderr == "error":
+            self.make_error(f"stderr was found in the cell outputs: {text}")
+        elif self.env.config.nb_output_stderr == "severe":
+            self.make_severe(f"stderr was found in the cell outputs: {text}")
+
+        if (
+            "remove-stderr" in self.node.metadata.get("tags", [])
+            or self.env.config.nb_output_stderr == "remove"
+        ):
             return []
 
-        container = nodes.container(classes=["stderr"])
-        container.append(
-            nodes.literal_block(
-                text=output["text"],
-                rawsource=output["text"],
-                language=self.env.config.nb_render_text_lexer,
-                classes=["stderr"],
-            )
+        node = nodes.literal_block(
+            text=output["text"],
+            rawsource=output["text"],
+            language=self.env.config.nb_render_text_lexer,
+            classes=["output", "stderr"],
         )
-        return [container]
+        return [node]
 
     def render_stdout(self, output: NotebookNode, index: int):
 
@@ -323,6 +348,7 @@ class CellOutputRenderer(CellOutputRendererBase):
 
     def render_text_latex(self, output: NotebookNode, index: int):
         text = output["data"]["text/latex"]
+        self.env.get_domain("math").data["has_equations"][self.env.docname] = True
         return [
             nodes.math_block(
                 text=strip_latex_delimiters(text),
@@ -378,7 +404,9 @@ class CellOutputRenderer(CellOutputRendererBase):
             # it becomes clickable?! (i.e. will open the image in the browser)
             image_node = nodes.image(uri=uri)
 
-            myst_meta_img = self.node.metadata.get(MYST_META_KEY, {}).get("image", {})
+            myst_meta_img = self.node.metadata.get(
+                self.env.config.nb_render_key, {}
+            ).get("image", {})
 
             for key, spec in [
                 ("classes", directives.class_option),
@@ -399,7 +427,9 @@ class CellOutputRenderer(CellOutputRendererBase):
                         )
                         return [self.make_error(error_msg)]
 
-            myst_meta_fig = self.node.metadata.get(MYST_META_KEY, {}).get("figure", {})
+            myst_meta_fig = self.node.metadata.get(
+                self.env.config.nb_render_key, {}
+            ).get("figure", {})
             if "caption" not in myst_meta_fig:
                 return [image_node]
 
@@ -472,6 +502,7 @@ class CellOutputRendererInline(CellOutputRenderer):
 
     def render_text_latex(self, output: NotebookNode, index: int):
         data = output["data"]["text/latex"]
+        self.env.get_domain("math").data["has_equations"][self.env.docname] = True
         return [
             nodes.math(
                 text=strip_latex_delimiters(data),
